import os
import requests
import json

class iData():

    def __init__(self):
        #initialize API tokens
        self.session_num = None
        self.session_token = None
        self.oauth_token = None
        self.auth_header = None
        self.base_URL = 'https://mygeohub.org/api'
        self.projects = dict()
        
        #get and save an OAUTH token
        self.get_auth_token()
        
        if self.oauth_token is not None:
            self.auth_header = {'Authorization': 'Bearer %s' % self.oauth_token}

        
    #makes a POST request with optional headers
    #returns the response object
    def make_post_request(self,url,data,header=None):
        r = requests.post(url,data,headers=header)
        return r
    
    #makes a GET request with optional headers
    #returns the response object
    def make_get_request(self,url,header=None):
        r = requests.get(url,headers=header)
        return r
        
    #sets the session ID and session token for use in obtaining a OAUTH authorization token
    def get_session_details(self):
        #resource file which stores the session ID and token
        resc_filename = '%s/resources' % os.environ['SESSIONDIR']
        resc_file = open(resc_filename,'r')
        for line in resc_file:
            data = line.split(' ')
            if len(data) == 2:
                if data[0] == 'sessionid':
                    self.session_num = data[1].rstrip()
                if data[0] == 'session_token':
                    self.session_token = data[1].rstrip()
        resc_file.close()
        
    #gets an OAUTH authorization token
    def get_auth_token(self):
        self.get_session_details()
        if self.session_num is not None and self.session_token is not None:
            oauth_request_data = {"grant_type":"tool",
                                  "sessionnum": self.session_num,
                                  "sessiontoken": self.session_token}
            oauth_request_url = "%s/developer/oauth/token" % self.base_URL
            resp = self.make_post_request(oauth_request_url,oauth_request_data)
            #check if request succeeded
            if resp.status_code == requests.codes.ok:
                #process response JSON
                resp_data = json.loads(resp.text)
                if 'access_token' in resp_data:
                    self.oauth_token = resp_data['access_token']
    
    #get the projects that the user has access to
    #stored in a dictionary keyed by project alias and contains the project ID
    #only for side-effect; ignore return value
    def collect_user_projects(self):
        if self.oauth_token is None:
            return None
        else:
            projects_url = '%s/projects/list' % self.base_URL
            resp = self.make_get_request(projects_url,self.auth_header)
            #check if request succeeded
            if resp.status_code == requests.codes.ok:
                #process JSON list of projects
                project_data = json.loads(resp.text)
                project_list = project_data['projects']
                #save the projects in a dictionary of alias and project ID
                for project in project_list:
                    if project['alias'] not in self.projects:
                        self.projects[project['alias']] = project['id']
            else:
                return None
    
    def get_project_id(self,project_alias):
        if project_alias in self.projects: #project data obtained previously
            return self.projects[project_alias]
        else: #we possibly never queried for the user's projects or this project was added subsequent to a query
            self.collect_user_projects()
            if project_alias in self.projects:
                return self.projects[project_alias]
            else:
                return None
               
    #we will always have a project_id since all API calls need it            
    def get_project_idata_connection_id(self,project_id):
        if self.oauth_token is None: #the init method should have got a token
            return None
        else:
            #make a request for the project's storage connections
            connections_url = '%s/projects/%s/files/connections' % (self.base_URL,project_id)
            resp = self.make_get_request(connections_url,self.auth_header)
            #check if request succeeded
            if resp.status_code == requests.codes.ok:
                #process JSON response
                connections_data = json.loads(resp.text)
                connections_list = connections_data['connections']
                for connection in connections_list:
                    #look for the iData storage connection
                    if connection['name'] == 'iData storage':
                        connection_id = connection['id']
                        return connection_id
                return None
            else:
                return None
    
    #gets metadata for a FUSE mounted iData file 
    def get_idata_file_metadata(self,filepath):
        #initialize result
        file_metadata = dict()
        #we only work with iData files from the FUSE mount
        if filepath.startswith("/srv/irods/"):
            #extract project alias and filepath
            project_path = filepath[len("/srv/irods/"):]
            filename = os.path.basename(project_path)
            project_path_parts = project_path.split('/')
            project_alias = project_path_parts[0]
            filedir_end = len(project_path) - len(filename) -1
            filedir_start = len(project_alias) + 1
            filedir = project_path[filedir_start:filedir_end]
            
            #if invalid path exit
            if len(project_alias) > 0:
                project_id = self.get_project_id(project_alias)
                if project_id is not None:
                    #get the iData storage connection ID for this project
                    connection_id = self.get_project_idata_connection_id(project_id)
                    if connection_id is not None:
                        #actual API call
                        metadata_request_url = '%s/projects/%s/files/connections/%s/getmetadata' % (self.base_URL,project_id,connection_id)
                        data = {'asset[]' : filename, 'subdir' : filedir}
                        resp = self.make_post_request(metadata_request_url,data,self.auth_header)
                        if resp.status_code == requests.codes.ok:
                            #process JSON response
                            resp_data = json.loads(resp.text)
                            #look for a metadata record for this file
                            if "metadata" in resp_data:
                                files_metadata = resp_data['metadata']
                                #API allows for metadata request on multiple files
                                #locate our record of interest
                                if filename in files_metadata:
                                    for key in files_metadata[filename]:
                                        file_metadata[key] = files_metadata[filename][key]['value']
        return file_metadata

    #sets metadata for a FUSE mounted iData file 
    #meta_kvdict is a dictionary of key-value pairs of metadata to set
    def set_idata_file_metadata(self,filepath,meta_kvdict):
        #we only work with iData files from the FUSE mount
        if filepath.startswith("/srv/irods/"):
            #extract project alias and filepath
            project_path = filepath[len("/srv/irods/"):]
            filename = os.path.basename(project_path)
            project_path_parts = project_path.split('/')
            project_alias = project_path_parts[0]
            filedir_end = len(project_path) - len(filename) -1
            filedir_start = len(project_alias) + 1
            filedir = project_path[filedir_start:filedir_end]
            
            #if invalid path exit
            if len(project_alias) > 0:
                project_id = self.get_project_id(project_alias)
                if project_id is not None:
                    #get the iData storage connection ID for this project
                    connection_id = self.get_project_idata_connection_id(project_id)
                    if connection_id is not None:
                        #actual API call
                        metadata_request_url = '%s/projects/%s/files/connections/%s/setmetadata' % (self.base_URL,project_id,connection_id)
                        data = {'asset[]' : filename, 'subdir' : filedir}
                        #add metadata 
                        for k,v in meta_kvdict.items():
                            data_key = 'metadata[%s]' % k
                            data[data_key] = v
                        resp = self.make_post_request(metadata_request_url,data,self.auth_header)
                        if resp.status_code == requests.codes.ok:
                            return True
                        else:
                            return False
        return False
